﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Documents;
using System.Windows;
using System.Windows.Controls;
using System.Security.Cryptography;
using System.Diagnostics;
//using System.Windows.Shapes;

namespace Practics.Model
{


    public class PathHelper
    {
        public static string GetPathTwoLevelsUp(string filePath)
        {
            // Получаем директорию, в которой находится файл
            string fileDirectory = Path.GetDirectoryName(filePath);

            // Получаем директорию на три уровня выше
            string pathThreeLevelsUp = Directory.GetParent(fileDirectory)?.Parent?.FullName;

            return pathThreeLevelsUp ?? throw new ArgumentException("Невозможно получить путь на три уровня выше.");
        }
    }

    //public class Problem
    //{
    //    public int ID { get; }
    //    public string ProblemName { get; }
    //    public string ProblemDescribe { get; }
    //    public string Chat { get; }

    //    public DateTime ProblemTime { get; }

    //    public DateTime ProblemSolvedTime { get; set; }

    //    public string ProblemSolvedDescribe { get; set; }

    //    public int ResposibleID { get; set; }

    //    public Problem()
    //    {

    //    }
    //}


    public class User
    {
        //Поля
        private int id;
        private string lastName;
        private string firstName;
        private string middleName;
        private int role;
        private string email;
        private string createAt;
        private string passwordHash;
        public int Role => role;
        public User(int id, string lastName, string firstName, string middleName, string email, string passwordHash, int role, DateTime createAt)
        {
            this.id = id;
            this.firstName = firstName;
            this.lastName = lastName;
            this.middleName = middleName;
            this.email = email;
            this.passwordHash = passwordHash;
            this.role = role;
        }


        public string GetShortName()
        {
            //return "";
            return $"{lastName} {firstName[0]}.{middleName[0]}.";
        }
        public string ShortName
        {
            get
            {
                //return "";
                return $"{lastName} {firstName[0]}.{middleName[0]}.";
            }


        }

        public int Id
        {
            get
            {
                return id;
            }
        }
    }

    //Делаем публичным для доступа из модуля тестов
    public class Model
    {

        private Model() //Защищаем от создание еще одного экземпляра Model
        {
            this.InitDataAdapterProblems();
            this.InitDataTableStatuses();
            this.InitDataTableProblemTypes();
            this.InitDataTableAdmins();
            this.InitDataTableKnowledge();

        }

        public static string HashPassword(string password)
        {
            using (SHA256 sha256 = SHA256.Create())
            {
                byte[] bytes = Encoding.UTF8.GetBytes(password);
                byte[] hash = sha256.ComputeHash(bytes);
                return Convert.ToBase64String(hash);
            }
        }

        static private Model? instance = null;

        //Синглтон
        public static Model ModelInstance
        {
            get
            {
                if (instance == null)
                    instance = new Model();
                return instance;
            }
        }


        static string currentDirectory = Directory.GetCurrentDirectory();
        public static string connectionString = "Data Source=(localdb)\\MSSQLLocalDB;AttachDbFilename=" + PathHelper.GetPathTwoLevelsUp(currentDirectory) + "\\dbtechsphere.mdf;Integrated Security=True;Connect Timeout=30;Encrypt=False";

        // Метод для проверки корректности email
        public bool CheckEmail(string email)
        {
            if (string.IsNullOrWhiteSpace(email))
                return false;

            // Регулярное выражение для проверки email
            string emailPattern = @"^[a-zA-Z0-9._%+-]+@(gmail\.com|mail\.ru|yandex\.ru)$";
            return Regex.IsMatch(email, emailPattern);
        }

        // Метод для проверки существования email
        public bool ExistsEmail(string email)
        {
            using (SqlConnection conn = new SqlConnection(connectionString))
            {
                conn.Open();
                string query = "SELECT COUNT(*) FROM Users WHERE email = @Email";
                SqlCommand command = new SqlCommand(query, conn);
                command.Parameters.AddWithValue("@Email", email);

                int count = (int)command.ExecuteScalar();
                return count > 0; // true, если email уже существует
            }
        }

        // Метод для проверки корректности пароля
        public bool CheckPassword(string password)
        {
            if (string.IsNullOrWhiteSpace(password) || password.Length < 6)
                return false;

            // Проверка наличия заглавных букв, строчных букв, цифр и спецсимволов
            bool hasUpperCase = password.Any(char.IsUpper);
            bool hasLowerCase = password.Any(char.IsLower);
            bool hasDigit = password.Any(char.IsDigit);
            bool hasSpecialChar = password.Any(ch => !char.IsLetterOrDigit(ch));

            return hasUpperCase && hasLowerCase && hasDigit && hasSpecialChar;
        }

        // Метод регистрации пользователя
        public bool Registration(string email, string password, string firstName, string lastName, string middleName, out string outMessage)
        {
            if (!CheckEmail(email))
            {
                outMessage = "Неправильная почта"; // Неправильный email
                return false;
            }

            if (!CheckPassword(password))
            {
                outMessage = "Неправильный пароль"; // Неправильный пароль
                return false;
            }

            if (ExistsEmail(email))
            {
                outMessage = "Данная почта уже существует"; // Email уже существует
                return false;
            }

            string hash = HashPassword(password);
            // Если все проверки пройдены, добавляем пользователя в базу данных
            AddUserToDB(email, hash, 1, firstName, lastName, middleName);

            outMessage = "Добро пожаловать"; // Регистрация успешна
            return true;
        }


        private void AddUserToDB(string email, string password, int role, string firstName, string lastName, string middleName)
        {
            //connectionString = ;// "Data Source=(localdb)\\MSSQLLocalDB;AttachDbFilename=" + currentDirectory + "\\dbtechsphere.mdf;Integrated Security=True;Connect Timeout=30;Encrypt=False";
            using (SqlConnection conn = new SqlConnection(connectionString))
            {
                conn.Open();
                // Преобразование в формат, подходящий для MS SQL
                string sqlDateTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");

                string query = "INSERT INTO Users (email, passwordHash, role, createAt, LastName, FirstName, MiddleName) Values (@Email, @Password, @Role, @CreateAt, @LastName, @FirstName, @MiddleName )";
                SqlCommand command = new SqlCommand(query, conn);

                // Добавляем параметры для предотвращения SQL-инъекций
                command.Parameters.AddWithValue("@Email", email);
                command.Parameters.AddWithValue("@Password", password);
                command.Parameters.AddWithValue("@Role", role);
                command.Parameters.AddWithValue("@CreateAt", sqlDateTime);
                command.Parameters.AddWithValue("@LastName", lastName);
                command.Parameters.AddWithValue("@FirstName", firstName);
                command.Parameters.AddWithValue("@MiddleName", middleName);

                int count = command.ExecuteNonQuery();
                System.Diagnostics.Debug.WriteLine($"Rows affected: {count}");
            }
        }


        public bool AddProblemToDB(int userID, string problemName, string pcNumber, string problemDescribe, int problemTypeID)
        {
            int count;
            using (SqlConnection conn = new SqlConnection(connectionString))
            {
                conn.Open();
                // Преобразование в формат, подходящий для MS SQL
                string sqlDateTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");

                string query = "INSERT INTO Problems (UserID, ProblemName, PCNumber, ProblemDescribe, ProblemTime, ProblemTypeID) Values (@UserID, @ProblemName, @PCNumber, @ProblemDescribe, @ProblemTime,@ProblemTypeID)";
                SqlCommand command = new SqlCommand(query, conn);

                // Добавляем параметры для предотвращения SQL-инъекций
                command.Parameters.AddWithValue("@UserID", userID);
                command.Parameters.AddWithValue("@ProblemName", problemName);
                command.Parameters.AddWithValue("@PCNumber", pcNumber);
                command.Parameters.AddWithValue("@ProblemDescribe", problemDescribe);
                command.Parameters.AddWithValue("@ProblemTime", sqlDateTime);
                command.Parameters.AddWithValue("@ProblemTypeID", problemTypeID);
                count = command.ExecuteNonQuery();
                System.Diagnostics.Debug.WriteLine($"Rows affected: {count}");
            }
            if (count > 0) return true; else return false;
        }
        public bool Login(string email, string password, out User user)
        {
            user = null;
            using (SqlConnection conn = new SqlConnection(connectionString))
            {
                conn.Open();
                // Преобразование в формат, подходящий для MS SQL
                string query = "SELECT * FROM Users Where email='" + email + "' and passwordHash='" + HashPassword(password) + "'";
                System.Diagnostics.Debug.WriteLine(query);
                SqlCommand command = new SqlCommand(query, conn);

                // Добавляем параметры для предотвращения SQL-инъекций

                SqlDataReader exists = command.ExecuteReader();
                int i = 0;
                int id = -1;
                string lastName = "";
                string firstName = "";
                string middleName = "";
                string passwordHash = "";
                string email2 = "";
                int role = 0;
                DateTime createAt = DateTime.Now;
                while (exists.Read())
                {
                    i++;
                    id = Convert.ToInt32(exists["id"]);
                    lastName = exists["LastName"].ToString();
                    firstName = exists["FirstName"].ToString();
                    middleName = exists["MiddleName"].ToString();
                    passwordHash = exists["passwordHash"].ToString();
                    email2 = exists["email"].ToString();
                    role = Convert.ToInt32(exists["role"]);
                    createAt = Convert.ToDateTime(exists["createAt"]);
                    System.Diagnostics.Debug.WriteLine($"{exists[0]} {exists[1]} {exists[2]} {exists[3]}");
                }
                System.Diagnostics.Debug.WriteLine($"Rows affected: {exists}");
                if (i == 1)
                {

                    user = new User(id, lastName, firstName, middleName, email2, passwordHash, role, createAt);
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        public SqlDataAdapter AdapterProblems;
        public DataTable DataTableProblems { get; } = new DataTable();

        public void UpdateProblems()
        {
            //System.Diagnostics.Debug.WriteLine(DataTableProblems.Columns[0].ColumnName);
            AdapterProblems.Update(DataTableProblems);
        }
        public void InitDataAdapterProblems()
        {
            SqlCommand sqlCommand = new SqlCommand("SELECT Problems.ID, Users.LastName,Users.FirstName,Users.MiddleName, Problems.ProblemName, Problems.PCNumber, Problems.ProblemDescribe, Problems.SolvedTime, Problems.ProblemStatusID, Problems.ProblemTime, Problems.ProblemTypeID, Problems.Chat, Problems.ResponsibleID, Problems.ProblemSolvedDescribe FROM Problems INNER JOIN Users ON Problems.UserID=Users.ID");
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            sqlCommand.Connection = conn;
            AdapterProblems = new SqlDataAdapter();
            AdapterProblems.SelectCommand = sqlCommand;
            sqlCommand = new SqlCommand(@"UPDATE Problems SET SolvedTime=@SolvedTime,  ProblemStatusID=@ProblemStatusID WHERE ID=@ID", conn);
            sqlCommand.Parameters.Add("@ID", SqlDbType.Int, 0, "ID");
            sqlCommand.Parameters[0].SourceVersion = DataRowVersion.Original;//становка SourceVersion = DataRowVersion.Original позволяет обновить данные, основываясь на оригинальном значении ключевого поля, а не на текущем изменённом значении. Это помогает избежать проблем при параллельном доступе к данным.
            sqlCommand.Parameters.Add("@SolvedTime", SqlDbType.DateTime, -1, "SolvedTime");
            //sqlCommand.Parameters.Add("@UserSolvedName", SqlDbType.Int, 0,"UserSolvedName");
            sqlCommand.Parameters.Add("@ProblemStatusID", SqlDbType.Int, 0, "ProblemStatusID");

            AdapterProblems.UpdateCommand = sqlCommand;
            AdapterProblems.Fill(DataTableProblems);

            System.Diagnostics.Debug.WriteLine(DataTableProblems.Columns.Count);
        }

        public void InitDataAdapterProblemsByFilter(DateTime? startTime, DateTime? finishTime, int problemStatusID, int problemTypeID)
        {
            DataTableProblems.Clear();

            string sqlTextString = "SELECT Problems.ID, Users.LastName,Users.FirstName,Users.MiddleName, Problems.ProblemName, Problems.PCNumber, Problems.ProblemDescribe, Problems.SolvedTime, Problems.ProblemStatusID, " +
                "Problems.ProblemTime, Problems.ProblemTypeID, Problems.Chat FROM Problems INNER JOIN Users ON Problems.UserID=Users.ID WHERE Problems.ProblemTime>=@StartTime AND  Problems.ProblemTime<=@FinishTime";
            SqlCommand sqlCommand;
            if (problemStatusID == -1) sqlTextString += " AND Problems.ProblemStatusID <> @ProblemStatusID";//выбираем все
            else sqlTextString += " AND Problems.ProblemStatusID = @ProblemStatusID";//выбираем выбранное
            if (problemTypeID == -1) sqlTextString += " AND Problems.ProblemTypeID <> @ProblemTypeID";
            else sqlTextString += " AND Problems.ProblemTypeID = @ProblemTypeID";

            sqlCommand = new SqlCommand(sqlTextString);
            SqlConnection conn;
            sqlCommand.Parameters.AddWithValue("@ProblemStatusID", problemStatusID);
            sqlCommand.Parameters.AddWithValue("@ProblemTypeID", problemTypeID);
            sqlCommand.Parameters.AddWithValue("@StartTime", startTime);
            sqlCommand.Parameters.AddWithValue("@FinishTime", finishTime);
            System.Diagnostics.Debug.WriteLine(sqlCommand.CommandText);
            conn = new SqlConnection(connectionString);
            sqlCommand.Connection = conn;
            AdapterProblems = new SqlDataAdapter();
            AdapterProblems.SelectCommand = sqlCommand;
            AdapterProblems.Fill(DataTableProblems);

            System.Diagnostics.Debug.WriteLine(DataTableProblems.Columns.Count);
        }


        public SqlDataAdapter AdapterProblemsByID;
        public DataTable DataTableProblemsByID;
        public void InitProblemsByUserID(int userID)
        {
            DataTableProblemsByID = new DataTable();
            SqlCommand sqlCommand = new SqlCommand("SELECT Problems.ID,  Users.LastName,Users.FirstName,Users.MiddleName, Problems.Chat, Problems.ProblemName, Problems.PCNumber, Problems.ProblemDescribe, Problems.SolvedTime, Problems.ProblemStatusID, Problems.ProblemTime, Problems.ResponsibleID FROM Problems INNER JOIN Users ON Problems.UserID=Users.ID WHERE Problems.UserID=" + userID.ToString());
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            sqlCommand.Connection = conn;
            //sqlCommand.Parameters.Add("@UserID", SqlDbType.Int, 0, userID.ToString());
            AdapterProblemsByID = new SqlDataAdapter();
            AdapterProblemsByID.SelectCommand = sqlCommand;
            /*
            sqlCommand = new SqlCommand(@"UPDATE Problems SET SolvedTime=@SolvedTime,  ProblemStatusID=@ProblemStatusID WHERE ID=@ID", conn);
            sqlCommand.Parameters.Add("@ID", SqlDbType.Int, 0, "ID");
            sqlCommand.Parameters[0].SourceVersion = DataRowVersion.Original;//становка SourceVersion = DataRowVersion.Original позволяет обновить данные, основываясь на оригинальном значении ключевого поля, а не на текущем изменённом значении. Это помогает избежать проблем при параллельном доступе к данным.
            sqlCommand.Parameters.Add("@UserID", SqlDbType.Int, 0, "UserID");
            sqlCommand.Parameters.Add("@SolvedTime", SqlDbType.DateTime, -1, "SolvedTime");
            //sqlCommand.Parameters.Add("@UserSolvedName", SqlDbType.Int, 0,"UserSolvedName");
            sqlCommand.Parameters.Add("@ProblemStatusID", SqlDbType.Int, 0, "ProblemStatusID");

            adapter.UpdateCommand = sqlCommand;
            */
            AdapterProblemsByID.Fill(DataTableProblemsByID);

            //System.Diagnostics.Debug.WriteLine(dataTable.Columns.Count);

            //return dataTable;
        }

        public void ChatUpdate(int id, string chatString)
        {
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            conn.Open();
            SqlCommand sqlCommand = new SqlCommand(@"UPDATE Problems SET Chat=@Chat WHERE ID=@ID", conn);
            //sqlCommand.Parameters.Add("@ID", SqlDbType.Int, -1, id.ToString());
            sqlCommand.Parameters.AddWithValue("@ID", id);
            sqlCommand.Parameters.AddWithValue("@Chat", chatString);
            sqlCommand.Parameters[0].SourceVersion = DataRowVersion.Original;//становка SourceVersion = DataRowVersion.Original позволяет обновить данные, основываясь на оригинальном значении ключевого поля, а не на текущем изменённом значении. Это помогает избежать проблем при параллельном доступе к данным.
            //sqlCommand.Parameters.Add("@Chat", SqlDbType.NVarChar, -1, chatString);
            //sqlCommand.Parameters.Add("@UserSolvedName", SqlDbType.Int, 0,"UserSolvedName");

            AdapterProblemsByID.UpdateCommand = sqlCommand;
            AdapterProblemsByID.Update(DataTableProblemsByID);

        }

        public void ChatUpdate2(int id, string chatString)
        {
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            conn.Open();
            SqlCommand sqlCommand = new SqlCommand(@"UPDATE Problems SET Chat=@Chat WHERE ID=@ID", conn);
            //sqlCommand.Parameters.Add("@ID", SqlDbType.Int, -1, id.ToString());
            sqlCommand.Parameters.AddWithValue("@ID", id);
            sqlCommand.Parameters.AddWithValue("@Chat", chatString);
            sqlCommand.Parameters[0].SourceVersion = DataRowVersion.Original;//становка SourceVersion = DataRowVersion.Original позволяет обновить данные, основываясь на оригинальном значении ключевого поля, а не на текущем изменённом значении. Это помогает избежать проблем при параллельном доступе к данным.
            //sqlCommand.Parameters.Add("@Chat", SqlDbType.NVarChar, -1, chatString);
            //sqlCommand.Parameters.Add("@UserSolvedName", SqlDbType.Int, 0,"UserSolvedName");

            AdapterProblems.UpdateCommand = sqlCommand;
            AdapterProblems.Update(DataTableProblems);
            conn.Close();

        }

        public void ProblemAdminUpdate(int id, DateTime? solvedTime, int problemStatusID, int responsibleID, string problemSolvedDescribe)
        {

            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            conn.Open();
            SqlCommand sqlCommand = new SqlCommand(@"UPDATE Problems SET SolvedTime=@SolvedTime, ProblemStatusID=@ProblemStatusID, ResponsibleID=@ResponsibleID, ProblemSolvedDescribe=@ProblemSolvedDescribe   WHERE ID=@ID", conn);
            //sqlCommand.Parameters.Add("@ID", SqlDbType.Int, -1, id.ToString());
            sqlCommand.Parameters.AddWithValue("@ID", id);
            //sqlCommand.Parameters.Add("@SolvedTime", SqlDbType.DateTime,0);
            sqlCommand.Parameters.AddWithValue("@SolvedTime", solvedTime==null?DBNull.Value: solvedTime?.ToString("yyyy-MM-dd HH:mm:ss"));
            sqlCommand.Parameters.AddWithValue("@ProblemStatusID", problemStatusID);
            sqlCommand.Parameters.AddWithValue("@ResponsibleID", responsibleID);
            sqlCommand.Parameters.AddWithValue("@ProblemSolvedDescribe", problemSolvedDescribe);
            sqlCommand.Parameters[0].SourceVersion = DataRowVersion.Original;//становка SourceVersion = DataRowVersion.Original позволяет обновить данные, основываясь на оригинальном значении ключевого поля, а не на текущем изменённом значении. Это помогает избежать проблем при параллельном доступе к данным.
            //sqlCommand.Parameters.Add("@Chat", SqlDbType.NVarChar, -1, chatString);
            //sqlCommand.Parameters.Add("@UserSolvedName", SqlDbType.Int, 0,"UserSolvedName");

            AdapterProblems.UpdateCommand = sqlCommand;
            AdapterProblems.Update(DataTableProblems);
            conn.Close();

        }





        public void RefillProblems()
        {
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            conn.Open();
            DataTableProblems.Clear();
            AdapterProblems.Fill(DataTableProblems);
            conn.Close();
        }

        public void RefillProblemsById()
        {
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            conn.Open();
            DataTableProblemsByID.Clear();
            AdapterProblemsByID.Fill(DataTableProblemsByID);
            conn.Close();
        }

        public DataTable DataTableStatuses { get; private set; } = new DataTable();
        public DataTable DataTableProblemTypes { get; private set; } = new DataTable();

        public void InitDataTableStatuses()
        {
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            SqlCommand sqlCommand = new SqlCommand("SELECT Id,name FROM Statuses");
            sqlCommand.Connection = conn;
            SqlDataAdapter tempAdapter = new SqlDataAdapter();
            tempAdapter.SelectCommand = sqlCommand;

            conn.Open();
            tempAdapter.Fill(DataTableStatuses);
            conn.Close();
            System.Diagnostics.Debug.WriteLine(DataTableStatuses.Columns[0].ColumnName);
        }


        public void InitDataTableProblemTypes()
        {
            DataTableProblemTypes.Clear();
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            SqlCommand sqlCommand = new SqlCommand("SELECT ID,NameProblemType FROM ProblemTypes");
            sqlCommand.Connection = conn;
            SqlDataAdapter tempAdapter = new SqlDataAdapter();
            tempAdapter.SelectCommand = sqlCommand;

            conn.Open();
            tempAdapter.Fill(DataTableProblemTypes);
            conn.Close();

        }


        public DataTable DataTableKnowledge { get; } = new DataTable();

        public SqlDataAdapter DataAdapterKnowledge;

        public void InitDataTableKnowledge()
        {
            DataTableKnowledge.Clear();
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            SqlCommand sqlCommand = new SqlCommand("SELECT Knowledge.ID, Knowledge.Note, Knowledge.TimeCreate, Knowledge.TimeUpdate,Knowledge.NoteName,Knowledge.UserID, Users.LastName  FROM Knowledge INNER JOIN Users ON Knowledge.UserID=Users.Id");
            sqlCommand.Connection = conn;
            DataAdapterKnowledge = new SqlDataAdapter();
            DataAdapterKnowledge.SelectCommand = sqlCommand;

            conn.Open();
            DataAdapterKnowledge.Fill(DataTableKnowledge);
            conn.Close();

        }


        public DataTable DataTableAdmins { get; } = new DataTable();

        public void InitDataTableAdmins()
        {
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            SqlCommand sqlCommand = new SqlCommand("SELECT ID, LastName FROM Users Where role=0");
            sqlCommand.Connection = conn;
            SqlDataAdapter tempAdapter = new SqlDataAdapter();
            tempAdapter.SelectCommand = sqlCommand;
            conn.Open();
            tempAdapter.Fill(DataTableAdmins);
            conn.Close();
        }

        public void LoadChatToRichTextBox(DataRow dataRow, RichTextBox richTextBox, string columnName = "Chat")
        {
            using (MemoryStream memoryStream = new MemoryStream())
            {
                using (StreamWriter writer = new StreamWriter(memoryStream))
                {
                    {
                        writer.Write(dataRow[columnName]);
                        writer.Flush();
                        memoryStream.Position = 0;
                        TextRange textRange = new TextRange(richTextBox.Document.ContentStart, richTextBox.Document.ContentEnd);
                        textRange.Load(memoryStream, DataFormats.Xaml);
                    }
                }
            }

        }

        public string RichTextBoxContentToString(RichTextBox richTextBox)
        {
            string documentContent;
            TextRange text = new TextRange(richTextBox.Document.ContentStart, richTextBox.Document.ContentEnd);
            using (MemoryStream memoryStream = new MemoryStream())
            {
                text.Save(memoryStream, DataFormats.Xaml);
                memoryStream.Position = 0;
                using (StreamReader streamReader = new StreamReader(memoryStream))
                {
                    documentContent = streamReader.ReadToEnd();
                }
                System.Diagnostics.Debug.WriteLine(documentContent);

            }
            return documentContent;
        }

        public void NoteUpdate(int id, string NoteString)
        {
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            conn.Open();
            SqlCommand sqlCommand = new SqlCommand(@"UPDATE Knowledge SET Note=@Note WHERE Id=@Id", conn);
            sqlCommand.Parameters.AddWithValue("@Id", id);
            sqlCommand.Parameters.AddWithValue("@Note", NoteString);
            sqlCommand.Parameters[0].SourceVersion = DataRowVersion.Original;//становка SourceVersion = DataRowVersion.Original позволяет обновить данные, основываясь на оригинальном значении ключевого поля, а не на текущем изменённом значении. Это помогает избежать проблем при параллельном доступе к данным.

            DataAdapterKnowledge.UpdateCommand = sqlCommand;
            DataAdapterKnowledge.Update(DataTableKnowledge);
            conn.Close();

        }

        public void NoteUpdateAll()
        {
            SqlConnection conn;
            conn = new SqlConnection(connectionString);
            conn.Open();
            SqlCommand sqlCommand = new SqlCommand(@"UPDATE Knowledge SET Note=@Note,NoteName=@NoteName, TimeCreate=@TimeCreate, TimeUpdate=@TimeUpdate WHERE Id=@Id", conn);
            sqlCommand.Parameters.Add("@Id", SqlDbType.Int, 0, "ID");
            sqlCommand.Parameters.Add("@Note", SqlDbType.NVarChar, 0, "Note");
            sqlCommand.Parameters.Add("@NoteName", SqlDbType.NVarChar, 0, "NoteName"); ;
            sqlCommand.Parameters.Add("@TimeCreate", SqlDbType.DateTime, 0, "TimeCreate");
            sqlCommand.Parameters.Add("@TimeUpdate", SqlDbType.DateTime, 0, "TimeUpdate");
            sqlCommand.Parameters[0].SourceVersion = DataRowVersion.Original;//становка SourceVersion = DataRowVersion.Original позволяет обновить данные, основываясь на оригинальном значении ключевого поля, а не на текущем изменённом значении. Это помогает избежать проблем при параллельном доступе к данным.
            DataAdapterKnowledge.UpdateCommand = sqlCommand;
            DataAdapterKnowledge.Update(DataTableKnowledge);
            conn.Close();

        }

        public bool InsertNote(int userID)
        {
            try
            {
                using (SqlConnection connection = new SqlConnection(connectionString))
                {
                    string insertQuery = "INSERT INTO Knowledge (TimeCreate, TimeUpdate, UserID) VALUES (@TimeCreate, @TimeUpdate, @UserID)";

                    SqlDataAdapter adapter = new SqlDataAdapter();
                    adapter.InsertCommand = new SqlCommand(insertQuery, connection);
                    adapter.InsertCommand.Parameters.AddWithValue("@UserID", userID);
                    adapter.InsertCommand.Parameters.AddWithValue("@TimeCreate", DateTime.Now);
                    adapter.InsertCommand.Parameters.AddWithValue("@TimeUpdate", DateTime.Now);

                    connection.Open();
                    adapter.InsertCommand.ExecuteNonQuery();
                    InitDataTableKnowledge();
                }
                return true;
            }
            catch (Exception ex) 
            {
                System.Diagnostics.Debug.WriteLine($"Ошибка при добавлении заметки:{ex.Message}");
                return false;
            }
        }


        public static bool ExportDataTableToCSV(DataTable dataTable, string filePath,out string outString)
        {
            try
            {
                // Создание StringBuilder для построения строки CSV
                StringBuilder sb = new StringBuilder();

                // Добавление заголовков столбцов
                sb.Append("Фамилия;");
                sb.Append("Имя;");
                sb.Append("Отчество;");
                sb.Append("Название проблемы;");
                sb.Append("Время решения;");
                sb.Append("Компьютер;");
                sb.Append("Статус;");
                sb.Append("Создано;");
                sb.Append("Ответственный");
                sb.AppendLine();

                int count = 0;
                // Добавление данных в строку CSV
                foreach (DataRow row in dataTable.Rows)
                {
                    count++;
                    //string problemStatus = "";
                    int problemStatusID = (int)(row["ProblemStatusID"]);
                    DataRow frow = ModelInstance.DataTableStatuses.Rows.Cast<DataRow>().FirstOrDefault(r => (int)r["ID"] == problemStatusID);
                    string nameStatus="None";
                    if (frow != null)
                    {
                        nameStatus = frow["Name"].ToString();
                    }
                    string responsibleName = "None";
                    if (!Convert.IsDBNull(row["ResponsibleID"]))
                    {
                        int responsibleID = (int)(row["ResponsibleID"]);
                        DataRow frow2 = ModelInstance.DataTableAdmins.Rows.Cast<DataRow>().FirstOrDefault(r => (int)r["ID"] == responsibleID);

                        if (frow2 != null)
                        {
                            responsibleName = frow2["LastName"].ToString();
                        }
                    }

                    sb.Append(row["LastName"].ToString()+";");
                    sb.Append(row["FirstName"].ToString() + ";");
                    sb.Append(row["MiddleName"].ToString() + ";");
                    sb.Append(row["ProblemName"].ToString() + ";");
                    sb.Append(row["SolvedTime"].ToString() + ";");
                    sb.Append(row["PCNumber"].ToString() + ";");
                    sb.Append(nameStatus+";");
                    sb.Append(row["ProblemTime"].ToString()+";");
                    sb.Append(responsibleName+";");
                    sb.AppendLine();
                }

                sb.AppendLine();
                sb.AppendLine($"Всего:;{count}");
                // Запись строки CSV в файл
                File.WriteAllText(filePath, sb.ToString(),Encoding.UTF8);
                //Process.Start("excel.exe", $"\"{filePath}\"");
                //Process.Start(Directory.GetCurrentDirectory());
                Process.Start("explorer.exe", $"\"{filePath}\"");
                outString=$"Файл сохранен под названием {filePath} в папке {Directory.GetCurrentDirectory()}";
                return true;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine($"Ошибка при экспорте в CSV-файл: {ex.Message}");
                outString=$"Ошибка при экспорте в CSV-файл: {ex.Message}";
                return false;
            }
            
        }


    }
}
